/**
 * @fileoverview &lt;label&gt; XML要素を探し、そのidとノンブルのリストをファイルに出力する
 * @author Kenshi Muto &lt;kmuto@debian.org&gt;
 * @requires glue code.jsx
 * @requires libGetFrameIDs.jsx
 * @requires libCommon.jsx
----------------------------------------------------------------------
ソフトウェア使用許諾同意書
本ソフトウェアの利用・変更・再配布にあたっては、下記の使用許諾同意書に
同意する必要があります。

1. 本使用許諾同意書における「ソフトウェア」とは、機械可読の資料 (ライブ
   ラリ、スクリプト、ソースファイル、データファイル)、実行形式、および
   文書を意味します。
2. 本ソフトウェアの使用許諾同意書に同意する限りにおいて、使用者は
   本ソフトウェアを自由に利用、変更することができます。
3. 本ソフトウェアに変更を加えない限りにおいて、使用者は本ソフトウェアを
   自由にコピー、再配布することができます。
4. 本ソフトウェアは無保証です。作者およびそれに関連する組織、配布者は、
   本ソフトウェアの使用に起因する一切の直接損害、間接損害、偶発的損害、
   特別損害、懲戒的損害、派生的損害について何らの責任・保証も負いません。
5. 本ソフトウェアを変更した上で再配布するときには、下記の事項すべてに
   従わなければなりません。
   - 使用許諾同意書の内容に変更を加えてはなりません。技術上の理由で
     文字エンコーディングの変換を行うことは許可しますが、その使用者が
     特殊な技術的措置なしに可読な形でなければなりません。
   - 技術上の理由でバイナリ化・難読化を行う場合も、変更箇所を含めた
     ソフトウェアを、その使用者が可読可能な形式の形で同一のメディアで
     提供しなければなりません。本使用許諾同意書の2条および3条により、
     使用者が可読形式の該当ソフトウェアを変更、コピー、再配布することを
     妨げてはなりません。
   - ソフトウェア構成物の所定の作者名の欄に、変更者のクレジット
     (個人名、企業名、所属、連絡先など)を「追加」しなければなりません。
6. 本ソフトウェアを変更した上で再配布するときには、変更理由および
   その内容を明記することが推奨されます。
7. 使用者がソフトウェアに適用可能な特許に対して特許侵害にかかわる何らか
   の行動を開始した時点で、この使用許諾同意書は自動的に終了し、以降
   使用者はこの使用許諾書によって与えられた一切の権利を放棄するものと
   します。

著作権所有者 Copyright (C) 2008-2013 Kenshi Muto. All rights reserved.
使用許諾同意書バージョン1.0
著作権所有者による書面での事前の許可がない限り、この使用許諾同意書
に変更を加えてはなりません。
----------------------------------------------------------------------
*/
#include "../libs/glue code.jsx"
#include "../libs/libGetFrameIDs.jsx"
#include "../libs/libCommon.jsx"

var __links = [];
var __p2s = null;

// 各行記述のファイルを開く
var filename = File.openDialog("目次ファイルリスト", "テキストファイル:*.txt,すべてのファイル:*" );
if (!filename) exit(0);

var fileObj = new File(filename);
if (fileObj.exists == false) exit(0);
fileObj.open("r");
while (!fileObj.eof) {
  filename = fileObj.readln();
  if (!filename.match(/^#/) && !filename.match(/^\s*$/)) {
    showInteraction(false);
    processDocumentForLabel(filename);
    showInteraction(true);
  }
}
fileObj.close();

var loop = 0;
do {
  var filename = File.saveDialog("ラベルリスト出力ファイル");
  if (filename) {
    var fileObj = new File(filename);
    fileObj.encoding = "UTF8";
    fileObj.lineFeed = "Unix";

    if (fileObj.open("w") == true) {
      for (var i = 0; i < __links.length; i++) {
	 fileObj.write(__links[i] + "\n");
      }
      fileObj.close();
      loop = false;
    } else {
      alert("ファイルを開けません。");
    }
  } else {
    loop = false;
  }
} while (loop == true);

function processDocumentForLabel(filename) {
  var file = new File(filename);
  if (file.exists == false) { // 存在チェック
    alert(filename + " というファイルがありません。");
    return false;
  }

  var document = app.open(file);

  if (document == null) {
    alert(filename + " を開くのに何かの問題が発生しました。処理を中止します。");
    exit(0);
  }

  processDocumentLabel(document);

  try {
    document.close(SaveOptions.no);
  } catch (e) {
    alert(filename + " をクローズできませんでした。");
  }
}

function processDocumentLabel(document) {
  var obj = new Object;
  obj.tagname = "label";
  obj.refattr = "id";

  __p2s = getPagenumFromSection(document);

  var myRuleSet = new Array(new processTakeLabel(obj));
  __processRuleSet(document.xmlElements.item(0), myRuleSet);

  obj.tagname = "wordlabel";
  var myRuleSet = new Array(new processTakeLabel(obj));
  __processRuleSet(document.xmlElements.item(0), myRuleSet);

  obj.tagname = "title";
  var myRuleSet = new Array(new processTakeLabel(obj));
  __processRuleSet(document.xmlElements.item(0), myRuleSet);
}

function processTakeLabel(obj) {
  this.name = "ProcessSubset";
  this.xpath = "//" + obj.tagname;
  this.apply = function(myElement, myRuleProcessor) {
    if (myElement.xmlAttributes.item(obj.refattr) == null) return true;
    var refval = myElement.xmlAttributes.item(obj.refattr).value;
    var page;
    try {
      page = myElement.storyOffset.parentTextFrames[0]
.parentPage.documentOffset;
    } catch (e) {
      page = myElement.storyOffset.parentTextFrames[0]
.parent.documentOffset;
    }
    __links.push(refval + "\t" + __p2s[page]);
    return true;
  }
}
